import _thread
from time import sleep
from src.wireless import espnow_send, espnow_recv, espnow_recv_raw
from lib.tm_sim import TuringMachine as TM
import src.data as data
import src.display as display

# placeholder in case somethin goes fucky wucky
tm: TM = TM("")

def __define(definition: str) -> None:
    global tm
    # its alive!
    tm = TM(definition)
    display.tm("", "", tm.get_state_current())
    display.draw()

    print(f"TM:CONFIG:Definition set")

def __band(band: str) -> None:
    global tm 
    # insert band here
    tm.set_band(band)
    display.tm(tm.get_band_context(3), "", tm.get_state_current())
    display.draw()
    print(f"TM:CONFIG:Band set")

def __step() -> None:
    global tm
    # step forth by one
    fin = tm.turing_step()
    display.tm(tm.get_band_context(3), tm.get_rule_used().replace(' ',''), tm.get_state_current())

    # master expects the used rule and current band
    send = tm.get_rule_used() + ":" + tm.get_band()
    espnow_send(data.master_mac, send)

    if fin:
        print(f"TM:STEP:Finished")
    else:
        print(f"TM:STEP:Step")

def __run() -> None:
    global tm
    # crank dat
    tm.turing_run()
    display.tm(tm.get_band_context(3), tm.get_rule_used().replace(' ',''), tm.get_state_current())

    espnow_send(data.master_mac, tm.get_band())
    print(f"TM:RUN:FINISHED")

def __pass() -> None:
    global tm
    # receive the address to which the band should be passed
    peer = espnow_recv_raw()
    # and do it
    espnow_send(peer, f'BAND:{tm.get_band()}')

def __result() -> None:
    global tm
    espnow_send(data.master_mac, tm.get_band())

def __reset() -> None:
    global tm
    # reset that sucker
    tm.turing_reset()
    display.tm("", "", tm.get_state_current())

    print(f"TM:CONFIG:RESET")

def __blink(pin) -> None:
    pin.value(0)
    sleep(.3)
    pin.value(1)
    sleep(.3)
    pin.value(0)
    sleep(.3)
    pin.value(1)


def loop(pin):
    display.setup()
    display.slave()
    display.draw()

    while True:
        msg = espnow_recv()

        # blinking in new _thread for faster execution
        # all below cant be done that way because of race conditions
        _thread.start_new_thread(__blink, (pin))

        if "DEFINE" in msg:
            __define(msg.split(':')[1])

        if "BAND" in msg:
            __band(msg.split(':')[1])

        if "STEP" in msg:
            __step()

        if "RUN" in msg:
            __run()

        if "PASS" in msg:
            __pass()

        if "RESULT" in msg:
            __result()

        if "RESET" in msg:
            __reset()

