from network import WLAN, AP_IF, STA_IF
from espnow import ESPNow
import src.data as data


def espnow_setup_master():
    global esp, wifi

    wifi = WLAN(AP_IF)
    wifi.active(True)
    wifi.config(ssid="wifiap", key="password")

    print(f"ESP:INIT:MASTER:Looking for Slaves")

    broadcast_mac = b'\xff' * 6
    esp.add_peer(broadcast_mac)

    slave_count = 1
    smsg = "ping"

    # ~30s max broadcasting to slaves
    # espnow limit is 20 peers
    for _ in range(0,30):
        espnow_send(broadcast_mac, smsg)
        print(f"MSG:SEND:STR:{smsg}")

        peer, rmsg = esp.recv(timeout_ms=1000)
        if peer:
            if type(rmsg) is bytes:
                print(f"MSG:RECV:STR:{rmsg.decode()}")

            espnow_send(peer, f"{slave_count}")
            data.slave_list.append(peer)
            slave_count += 1
            
            print(f"ESP:INIT:MASTER:Found slave")
    
    dedup = []
    [dedup.append(x) for x in data.slave_list if x not in dedup]
    data.slave_list = dedup
    
    print(f"ESP:INIT:MASTER:Found {len(data.slave_list)} slaves")


def espnow_setup():
    global esp

    wifi = WLAN(STA_IF)
    wifi.active(True)

    esp = ESPNow()
    esp.active(True)

    print(f"ESP:INIT:SLAVE:Looking for Master")

    # 20s wait time for message
    host, msg = esp.recv(timeout_ms=20000)
    if host:
        data.master = False
        data.master_mac = host

        # has to be, still need this
        if type(msg) is bytes:
            print(f"MSG:RECV:STR:{msg.decode()}")

        espnow_send(data.master_mac, "pong")
        # ignore repeat pings and wait for correct package
        while True:
            try:
                data.slave_num = int(espnow_recv())
                break
            except:
                pass

        print(f"ESP:INIT:SLAVE:Found Master, registered as slave {data.slave_num}")

    else:
        data.master = True
        espnow_setup_master()


def espnow_send(peer: bytes, msg: str) -> bool:
    """
    Returns True if peer has received the message.
    Intended to be used with espnow_recv.
    """
    print(f"MSG:SEND:STR:{msg}")
    if not peer in [x[0] for x in esp.get_peers()] and not b'\xff'*6 in peer:
        esp.add_peer(peer)
    return esp.send(peer, msg.encode())


def espnow_send_raw(peer: bytes, msg: bytes) -> bool:
    """
    Returns True if peer has received the message.
    Intended to be used with espnow_recv_raw.
    """
    print(f"MSG:SEND:RAW:{msg}")
    if not peer in [x[0] for x in esp.get_peers()]:
        esp.add_peer(peer)
    return esp.send(peer, msg)


def espnow_recv() -> str:
    """
    Returns the received, decoded message.
    Locks up further execution until a message is received.
    """
    timeout = 5000 # 5s

    host, msg = esp.recv(timeout)
    while not host:
        host, msg = esp.recv(timeout)

    if type(msg) is bytes:
        print(f"MSG:RECV:STR:{msg.decode()}")
        return msg.decode()
    else:
        return ''


def espnow_recv_raw() -> bytes:
    """
    Returns the received message without decoding.
    Locks up further execution until a message from master is received.
    """
    timeout = 5000 # 5s

    host, msg = esp.recv(timeout)
    while not host:
        host, msg = esp.recv(timeout)

        if host != data.master_mac:
            print(f"MSG:RECV:ERR:Message not from Master")
            host = None

    print(f"MSG:RECV:RAW:{msg}")

    # msg has to be bytes but still needs to be checked here
    if type(msg) is bytes:
        return msg
    else:
        return b''

